% function TL=model(l,b,ITx,IRx,Iidle,Enode,D,M):
%    TL...lifetime network
%     l...size of the transmitted message
%     b...bit rate
%   ITx...current consumption in mode broadcasting
%   IRx...current consumption in mode receiving
% Iidle...current consumption in mode idle
% Enode...total energy node
%     D...interval broadcasts
%     M...network error
%% =====================================

function TL=model(l,b,ITx,IRx,Iidle,Enode,D,M)
    load nodesM.mat
    R=150;
    numNodes=48;
    receiver=25;
    Enodemin=13499;		%spodn hranice energie uzlu 
    U=1.5;
    Rxack=0.16e-3;
    txdata=(l*8)/(b*1000);                                                    
    Eidle=(U*Iidle*(D-4e-3));     
    ACK=5e-3;
    
%% =============Main================
[netM,RxTxM,LT]=create_netM(numNodes,nodesM,Enode);
E=printNet(R,netM);
[Distance_Zone,Distance_Time_Help]=create_Distance(numNodes,receiver,E);
[TL,RxTxM,LT]=simulation(netM,R,E,receiver,Enodemin,RxTxM,Rxack,numNodes,txdata,ACK,U,ITx,IRx,Eidle,D,M,LT,Distance_Zone,Distance_Time_Help)
%print_TimeLive(numNodes,LT,Distance_Zone,Distance_Time_Help)

%% =============Functions================         
function [netM,RxTxM,LT]=create_netM(numNodes,nodesM,Enode)
ID=1;
for i=1:numNodes
    netM(1,ID)=ID;% inicializaec topologie
    RxTxM(1,ID)=ID; % inicializace matice RxTxM
    netM(2,ID)=nodesM(1,ID);
    netM(3,ID)=nodesM(2,ID);
    RxTxM(2,ID)=0;
    RxTxM(3,ID)=0;
    RxTxM(4,ID)=Enode;
    ID=ID+1;               
end
L=1;
for i=1:numNodes-1
    LT(1,L)=L;
    L=L+1;               
end

function [Distance_Zone,Distance_Time_Help]=create_Distance(numNodes,receiver,E)
Zone=0;
Min=0;
Max=0;
for j=1:numNodes
    sender=j;
    if sender~=receiver       
        sp=shortestPath(E,sender,receiver);
        Zone=numel(sp);
        Distance_Zone(1,sender)=sender;
        Distance_Zone(2,sender)=Zone;
    end
end
Distance_Zone(:,25) = [];
Min=min(min(Distance_Zone(2,:)));
Max=max(max(Distance_Zone(2,:)));
for j=Min:Max
    zone=j;
    Distance_Time_Help(1,zone)=zone;      
end
Distance_Time_Help(:,1) = [];

function E=printNet(R,netM) 
    I = imread('mapa.png');
    imshow(I);
    hold on;     
    plot(netM(2,:),netM(3,:),'bo','MarkerSize',6,'MarkerFaceColor','b');   
    hold all;
    radek=1;
    for j=1:numel(netM(1,:))
        for jTemp=1:numel(netM(1,:))
         X1=netM(2,j);
         Y1=netM(3,j);
         X2=netM(2,jTemp);
         Y2=netM(3,jTemp);
         xSide=abs(X2-X1);
         ySide=abs(Y2-Y1);
         d=sqrt(xSide^2+ySide^2);
         if (d<R)&&(j~=jTemp)
             vertice1=[X1,X2];
             vertice2=[Y1,Y2];
             plot(vertice1,vertice2,'-r','LineWidth',0.4);
             hold all;
             E(radek,1)=j;
             E(radek,2)=jTemp;
             E(radek,3)=d;
             radek=radek+1;
         end
        end
    end
    v=netM(1,:);
    vv=v';
    s=int2str(vv);
    text(netM(2,:)+1,netM(3,:)+3,s,'FontSize',10,'VerticalAlignment','Baseline');
    hold all; 
  
    %% Shortest Path Establishment
function sp=shortestPath(E,sender,receiver)
    [dSP,sp]=grShortPath(E,sender,receiver);
      
%% Records unicast transmission into the TxRx matrix
function [RxTxM,sp]=unicast(E,RxTxM,Rxack,sender,receiver,txdata,U,ITx,IRx,Eidle)
    sp=shortestPath(E,sender,receiver);
    BE = randi(3,1,1);
    CCA=(2^BE-1)*320e-6+128e-6;                         
    ttotal=CCA+txdata+Rxack;                           
    Esent=(U*ITx*ttotal);                   
    ET=Eidle+Esent;
    ER=(U*IRx*ttotal);
    Etotal=ET+ER;
        for j=1:numel(sp)
            node=sp(j);
            if node == receiver
                if j==1
                    RxTxM(3,node)=RxTxM(3,node)+1;
                elseif j==numel(sp)
                    RxTxM(2,node)=RxTxM(2,node)+1;
                else
                RxTxM(2,node)=RxTxM(2,node)+1;
                RxTxM(3,node)=RxTxM(3,node)+1;
                end
            else
                if j==1
                    RxTxM(3,node)=RxTxM(3,node)+1;
                    RxTxM(4,node)=RxTxM(4,node)-ET;
                elseif j==numel(sp)
                    RxTxM(2,node)=RxTxM(2,node)+1;
                    RxTxM(4,node)=RxTxM(4,node)-ER;
                else
                RxTxM(2,node)=RxTxM(2,node)+1;
                RxTxM(3,node)=RxTxM(3,node)+1;
                RxTxM(4,node)=RxTxM(4,node)-Etotal;
                end
            end
        end
        
function [RxTxM,sp]=unicastbroken(E,RxTxM,Rxack,sender,receiver,receiverbroken,txdata,ACK,U,ITx,IRx,Eidle)
    sp=shortestPath(E,sender,receiverbroken);
    BE = randi(3,1,1);
    CCA=(2^BE-1)*320e-6+128e-6;                         
    ttotalbroken=CCA+txdata+ACK+CCA+txdata+Rxack;                           
    Esent=(U*ITx*ttotalbroken);%*1000;                       
    ET=Eidle+Esent;
    ER=(U*IRx*ttotalbroken);%*1000; 
    Etotal=ET+ER;
         for j=1:numel(sp)
            node=sp(j);
          if node == receiver
            if j==1
                RxTxM(3,node)=RxTxM(3,node)+1;
            elseif j==numel(sp)
                RxTxM(2,node)=RxTxM(2,node)+1;
            else
            RxTxM(2,node)=RxTxM(2,node)+1;
            RxTxM(3,node)=RxTxM(3,node)+1;
            end
          else
              if j==1
                RxTxM(3,node)=RxTxM(3,node)+1;
                RxTxM(4,node)=RxTxM(4,node)-ET;
            elseif j==numel(sp)
                RxTxM(2,node)=RxTxM(2,node)+1;
                RxTxM(4,node)=RxTxM(4,node)-ER;
              else
            RxTxM(2,node)=RxTxM(2,node)+1;
            RxTxM(3,node)=RxTxM(3,node)+1;
            RxTxM(4,node)=RxTxM(4,node)-Etotal;
              end
          end       
         end
        
function print_TimeLive(numNodes,LT,Distance_Zone,Distance_Time_Help)
node=0;
zone=0;
row=2;
for j=1:numNodes
    sender=j;
    if sender~=25
        [RO,CO] = find(LT(2,:)==sender);
        node=LT(3,CO);
        [RO,CO] = find(Distance_Zone(1,:)==sender);
        zone=Distance_Zone(2,CO);
        [RO,CO] = find(Distance_Time_Help(1,:)==zone);
        Distance_Time_Help(row,CO)=node;
    end
    row=row+1;
end
Zone_Time=Distance_Time_Help(2:49,:);
Zone_Time(Zone_Time==0)=NaN;
figure;
boxplot(Zone_Time,Distance_Time_Help(1,:));
xlabel('Zny');
ylabel('ivotnost ve dnech');
title('ivotnost st');

function [TL,RxTxM,LT]=simulation(netM,R,E,receiver,Enodemin,RxTxM,Rxack,numNodes,txdata,ACK,U,ITx,IRx,Eidle,D,M,LT,Distance_Zone,Distance_Time_Help)
sp=1;
number_of_rounds=1;
month=(30*1440)/D;
repeat_month=month;
selfdischarge_Energy=25;
Mat=zeros;
Time=zeros;
E=printNet(R,netM);
while numel(Mat) ~= 48
    for j=1:numNodes
            sender=j;
            if sender~=receiver
                sp=shortestPath(E,sender,receiver);
                    if numel(sp) ~= 0
                        N = randi(50,1,1);
                    			if N < M						%chybovost                               
                        			sp=shortestPath(E,sender,receiver);	%najit cesty k pjemci
                        			k = numel(sp);
                        			if k == 0
                            				sp=shortestPath(E,sender,receiver);
                        			else
                            				h = randi(k,1,1);			%nov pjemce
                            				if h == 1
                                				receiverbroken=sp(h);
                                				[RxTxM,sp]=unicastbroken(E,RxTxM,Rxack,sender,receiver,receiverbroken,txdata,ACK,U,ITx,IRx,Eidle);
                            				else
                                				receiverbroken=sp(h-1);
                                				[RxTxM,sp]=unicastbroken(E,RxTxM,Rxack,sender,receiver,receiverbroken,txdata,ACK,U,ITx,IRx,Eidle);
                            				end 
                         			end
                                end
                        [RxTxM,sp]=unicast(E,RxTxM,Rxack,sender,receiver,txdata,U,ITx,IRx,Eidle);
                            if find(RxTxM(4,:) <= Enodemin) 
                                [ROW,COL] = find(RxTxM(4,:) <= Enodemin);				
                                DEL=setdiff(COL,Mat);							  
                                Mat=[Mat DEL];
                                numel(DEL);
                                if DEL ~= 0
                                    LTime=(number_of_rounds*D)/1440;
                                    for j=1:numel(DEL)
                                        Time=[Time LTime];
                                    end
                                    netM(2,DEL)=-500;
                                    netM(3,DEL)=-500;
                                    E=printNet(R,netM);
                                end 
                            end
                    else
                        del2=sender;
                        delnode2=del2;
                        DEL=setdiff(delnode2,Mat);
                        Mat=[Mat DEL];
                        numel(DEL);
                        if DEL ~= 0
                            LTime=(number_of_rounds*D)/1440;
                            for j=1:numel(DEL)
                                Time=[Time LTime];
                            end
                            netM(2,DEL)=-500;
                            netM(3,DEL)=-500;
                            E=printNet(R,netM);
                        end
                    end
            end
            %selfdischarge=(number_of_rounds*D)/1440;
    end
       if number_of_rounds==month
                for j=1:numNodes
                    sender=j;
                    if sender~=receiver
                        RxTxM(4,sender)=RxTxM(4,sender)-selfdischarge_Energy;
                    end
                end
                RxTxM
                month=month+repeat_month;
       end
    number_of_rounds=number_of_rounds+1;
    numel(Mat);
end
Mat(:, 1) = [];
Time(:, 1) = [];
LT=[LT; Mat; Time];
print_TimeLive(numNodes,LT,Distance_Zone,Distance_Time_Help);
TL=(number_of_rounds*D)/1440;	%as ve dnech